# -*- coding: utf-8 -*-
# @Time    : 2020/7/6 16:09
# @Author  : lightsmile
# @Software: PyCharm

import platform
import subprocess

encode_dict = {
    'Windows': 'gbk',
    'Linux': 'utf8'
}


def execute_cmd(cmd: str):
    try:
        process = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE, stderr=subprocess.PIPE)
        cmd_output = process.stdout.read().decode(encode_dict[platform.system()])
        return True, cmd_output
    except Exception:
        return False, None


if __name__ == '__main__':
    state, info = execute_cmd('ls')
    if state:
        print(info)



import json
import os


def read_json_line(filename: str, error_file: str = None):
    """
    从文件中逐行获取json对象
    :param filename: json文件名
    :param error_file: 保存错误信息的文件路径，若不指定则系统默认当前执行路径下`error.log`文件
    :return: json object 迭代器
    """
    if not error_file:
        error_file = os.path.abspath('.') + os.sep + 'error.log'
    with open(filename, encoding='utf-8') as f:
        line_count = 0
        for line in f:
            line_count += 1
            try:
                json_line = json.loads(line.strip())
                yield json_line
            except Exception as e:
                with open(error_file, 'a+', encoding='utf-8') as err_f:
                    err_f.write("line{}: {}".format(line_count, line))
                    continue


def write_json_line(file_obj, json_obj):
    """
    将数据对象以json格式写入到文件中
    :param file_obj: 文件对象
    :param json_obj: 可以被序列化为json字符串的数据对象
    :return:
    """
    file_obj.write(json.dumps(json_obj, ensure_ascii=False))
    file_obj.write('\n')



__all__ = ['send_email_notification']

import yagmail

yag = yagmail.SMTP(user="lightfight@qq.com", password="bvxvmqapqqpideah", host="smtp.qq.com", port="465")


def send_email_notification(to, subject, contents):
    """
    a function which can send an email
    :param to: the mailbox of recipient
    :param subject: the subject of the email
    :param contents: the content of email
    :return: True or False
    """
    try:
        yag.send(to=to, subject=subject, contents=contents)
        return True
    except Exception as e:
        return False



# -*- coding: utf-8 -*-
# @Time    : 2020/6/29 15:04
# @Author  : lightsmile
# @Software: PyCharm


def batch(iterable, n=10):
    """
    将迭代器转换为返回批量数据的形式
    :param iterable: 待转换迭代器
    :param n: 每批数据容量
    :return: 批量数据迭代器
    """
    iterable = iter(iterable)
    while True:
        chunk = []
        for i in range(n):
            try:
                chunk.append(next(iterable))
            except StopIteration:
                if chunk:
                    yield chunk
                return
        yield chunk


def cutoff_iter(iterable, n=10):
    """
    将迭代器截断只返回前面一部分，类似linux中的head命令
    :param iterable:
    :param n:
    :return:
    """
    iterable = iter(iterable)
    try:
        for i in range(n):
            yield next(iterable)
    except StopIteration:
        return



# -*- coding: utf-8 -*-
import os
import sys

from ..common.file import get_file_name
from ..common.log import logger


def add_sys_path(file_path: str, project_name: str):
    if not os.path.exists(file_path):
        raise FileNotFoundError("{} not found".format(file_path))
    flag = False
    parent_path = os.path.abspath(file_path)
    parent_name = get_file_name(parent_path)
    project_path = None
    while parent_name:
        parent_path = os.path.dirname(parent_path)
        for child_name in os.listdir(parent_path):
            if child_name == project_name:
                flag = True
                project_path = parent_path
                break
        if flag:
            break
    if flag:
        sys.path.insert(0, project_path)
        logger.info("已成功将{}添加至系统路径".format(project_path))
    else:
        raise FileNotFoundError("{} not found".format(project_name))




# -*- coding: utf-8 -*-
# @Time    : 2020/7/20 18:36
# @Author  : lightsmile
# @Software: PyCharm

# author: [bojone](https://github.com/bojone)
# source code link: [text_compare/compare.py](https://github.com/bojone/text_compare/blob/master/compare.py)
from .string import longest_common_subsequence


def red_color(text):
    return u'\033[1;31m%s\033[0m' % text


def green_color(text):
    return u'\033[32m%s\033[0m' % text


def blue_color(text):
    return u'\033[1;34m%s\033[0m' % text


def string_compare(source, target):
    _, mapping = longest_common_subsequence(source, target)
    source_idxs = set([i for i, j in mapping])
    target_idxs = set([j for i, j in mapping])
    colored_source, colored_target = u'', u''
    for i, j in enumerate(source):
        if i in source_idxs:
            colored_source += green_color(j)
        else:
            colored_source += red_color(j)
    for i, j in enumerate(target):
        if i in target_idxs:
            colored_target += green_color(j)
        else:
            colored_target += blue_color(j)
    print(colored_source)
    print(colored_target)



# -*- coding: utf-8 -*-
# @Time    : 2020/7/20 18:30
# @Author  : lightsmile
# @Software: PyCharm

from collections import defaultdict


# author: [bojone](https://github.com/bojone)
# source code link: [bert4keras/snippets.py](https://github.com/bojone/bert4keras/blob/master/bert4keras/snippets.py)
def longest_common_substring(source, target):
    """最长公共子串（source和target的最长公共切片区间）
    返回：子串长度, 所在区间（四元组）
    注意：最长公共子串可能不止一个，所返回的区间只代表其中一个。
    """
    c, l, span = defaultdict(int), 0, (0, 0, 0, 0)
    for i, si in enumerate(source, 1):
        for j, tj in enumerate(target, 1):
            if si == tj:
                c[i, j] = c[i - 1, j - 1] + 1
                if c[i, j] > l:
                    l = c[i, j]
                    span = (i - l, i, j - l, j)
    return l, span


# author: [bojone](https://github.com/bojone)
# source code link: [bert4keras/snippets.py](https://github.com/bojone/bert4keras/blob/master/bert4keras/snippets.py)
def longest_common_subsequence(source, target):
    """最长公共子序列（source和target的最长非连续子序列）
    返回：子序列长度, 映射关系（映射对组成的list）
    注意：最长公共子序列可能不止一个，所返回的映射只代表其中一个。
    """
    c = defaultdict(int)
    for i, si in enumerate(source, 1):
        for j, tj in enumerate(target, 1):
            if si == tj:
                c[i, j] = c[i - 1, j - 1] + 1
            elif c[i, j - 1] > c[i - 1, j]:
                c[i, j] = c[i, j - 1]
            else:
                c[i, j] = c[i - 1, j]
    l, mapping = c[len(source), len(target)], []
    i, j = len(source) - 1, len(target) - 1
    while len(mapping) < l:
        if source[i] == target[j]:
            mapping.append((i, j))
            i, j = i - 1, j - 1
        elif c[i + 1, j] > c[i, j + 1]:
            j = j - 1
        else:
            i = i - 1
    return l, mapping[::-1]



# -*- coding: utf-8 -*-
# @Time    : 2020/7/7 8:36
# @Author  : lightsmile
# @Software: PyCharm

import os
from typing import Union, List


class NotSpecifiedFileException(Exception):
    def __init__(self, info: Union[str, List[str]]):
        if isinstance(info, str):
            self._info = info
        else:
            self._info = '/'.join(info)

    def __str__(self):
        return "并非指定的文件错误，应该是形如：xx.{}".format(self._info)


def check_file(file_path: str, ext: Union[str, List[str]] = None):
    if not os.path.exists(file_path):
        raise FileNotFoundError
    if ext:
        if isinstance(ext, str) and not file_path.endswith(ext):
            raise NotSpecifiedFileException(ext)
        if isinstance(ext, list):
            file_ext = os.path.splitext(file_path)[-1]
            if file_ext:
                file_ext = file_ext[1:]
            if file_ext not in ext:
                raise NotSpecifiedFileException(ext)


def get_file_name(file_path: str):
    """
    获取文件名，例如：
    :param file_path: fuck/fuck/fuck.txt -> fuck
    :return: 文件名，不含拓展名
    """
    _, name = os.path.split(file_path)
    file_name, _ = os.path.splitext(name)
    return file_name


if __name__ == '__main__':
    # check_file('fuck.txt', 'txt')
    # check_file('__init__.py', 'txt')
    check_file('__init__.py', 'py')
    check_file('__init__.py', ['py', 'pyw'])
    print(get_file_name('hello_world.py'))



from distutils.core import setup
import setuptools

with open('./version.txt', encoding='utf8') as f:
    version = f.read().strip()

with open('./README.md', 'r', encoding='utf8') as f:
    long_description = f.read()

with open('./requirements.txt', 'r', encoding='utf8') as f:
    install_requires = list(map(lambda x: x.strip(), f.readlines()))

setup(
    name='lightUtils',
    version=version,
    description="lightsmile's personal Python util libs",
    author='lightsmile',
    author_email='iamlightsmile@gmail.com',
    url='https://github.com/smilelight/lightUtils',
    packages=setuptools.find_packages(),
    install_requires=install_requires,
    long_description=long_description,
    long_description_content_type='text/markdown',
    license='MIT',
    classifiers=[
        'Development Status :: 4 - Beta',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Libraries'
    ],
)



from .json_file import read_json_line, write_json_line
from .notification import send_email_notification
from .yaml_file import load_yaml
__all__ = ['read_json_line', 'send_email_notification', 'write_json_line', 'load_yaml']



# -*- coding: utf-8 -*-
# @Time    : 2020/7/17 14:13
# @Author  : lightsmile
# @Software: PyCharm
import inspect

from .log import logger


def retrieve_name(var, more_call=False):
    """
    get a name of variables
    :param more_call: 是否是嵌套调用该函数，如果嵌套调用的话，需要再往函数调用栈返一层才能获取变量名了。
    :param var: the variable
    :return: the name of variable
    """
    if more_call:
        callers_local_vars = inspect.currentframe().f_back.f_back.f_locals.items()
    else:
        callers_local_vars = inspect.currentframe().f_back.f_locals.items()
    var_names = [var_name for var_name, var_val in callers_local_vars if var_val is var]
    if not var_names:
        return None
    return var_names[0]


def inspect_variable(var):
    var_name = retrieve_name(var, more_call=True)
    return {
        'name': var_name,
        'type': str(type(var)),
        'attrs': str(dir(var))
    }



#!/usr/bin/env python3
# -*- coding:utf-8 -*-

"""
    log.py
    ~~~~~~

    log module

    :author:    lightless <root@lightless.me>
    :homepage:  None
    :license:   GPL-3.0, see LICENSE for more details.
    :copyright: Copyright (c) 2017 lightless. All rights reserved
"""

import os
import logging
import logging.handlers

__all__ = ["logger"]


# 用户配置部分 ↓
LEVEL_COLOR = {
    'DEBUG': 'cyan',
    'INFO': 'green',
    'WARNING': 'yellow',
    'ERROR': 'red',
    'CRITICAL': 'red,bg_white',
}
STDOUT_LOG_FMT = "%(log_color)s[%(asctime)s] [%(levelname)s] [%(threadName)s] [%(filename)s:%(lineno)d] %(message)s"
STDOUT_DATE_FMT = "%Y-%m-%d %H:%M:%S"
FILE_LOG_FMT = "[%(asctime)s] [%(levelname)s] [%(threadName)s] [%(filename)s:%(lineno)d] %(message)s"
FILE_DATE_FMT = "%Y-%m-%d %H:%M:%S"
# 用户配置部分 ↑


class ColoredFormatter(logging.Formatter):

    COLOR_MAP = {
        "black": "30",
        "red": "31",
        "green": "32",
        "yellow": "33",
        "blue": "34",
        "magenta": "35",
        "cyan": "36",
        "white": "37",
        "bg_black": "40",
        "bg_red": "41",
        "bg_green": "42",
        "bg_yellow": "43",
        "bg_blue": "44",
        "bg_magenta": "45",
        "bg_cyan": "46",
        "bg_white": "47",
        "light_black": "1;30",
        "light_red": "1;31",
        "light_green": "1;32",
        "light_yellow": "1;33",
        "light_blue": "1;34",
        "light_magenta": "1;35",
        "light_cyan": "1;36",
        "light_white": "1;37",
        "light_bg_black": "100",
        "light_bg_red": "101",
        "light_bg_green": "102",
        "light_bg_yellow": "103",
        "light_bg_blue": "104",
        "light_bg_magenta": "105",
        "light_bg_cyan": "106",
        "light_bg_white": "107",
    }

    def __init__(self, fmt, datefmt):
        super(ColoredFormatter, self).__init__(fmt, datefmt)

    def parse_color(self, level_name):
        color_name = LEVEL_COLOR.get(level_name, "")
        if not color_name:
            return ""

        color_value = []
        color_name = color_name.split(",")
        for _cn in color_name:
            color_code = self.COLOR_MAP.get(_cn, "")
            if color_code:
                color_value.append(color_code)

        return "\033[" + ";".join(color_value) + "m"


    def format(self, record):
        record.log_color = self.parse_color(record.levelname)
        message = super(ColoredFormatter, self).format(record) + "\033[0m"

        return message


def _get_logger(log_to_file=True, log_filename="default.log", log_level="DEBUG"):

    _logger = logging.getLogger(__name__)

    stdout_handler = logging.StreamHandler()
    stdout_handler.setFormatter(
        ColoredFormatter(
            fmt=STDOUT_LOG_FMT,
            datefmt=STDOUT_DATE_FMT,
        )
    )
    _logger.addHandler(stdout_handler)

    if log_to_file:
        _tmp_path = os.path.dirname(os.path.abspath(__file__))
        _tmp_path = os.path.join(_tmp_path, "../logs/{}".format(log_filename))
        file_handler = logging.handlers.TimedRotatingFileHandler(_tmp_path, when="midnight", backupCount=30)
        file_formatter = logging.Formatter(
            fmt=FILE_LOG_FMT,
            datefmt=FILE_DATE_FMT,
        )
        file_handler.setFormatter(file_formatter)
        _logger.addHandler(file_handler)

    _logger.setLevel(log_level)
    return _logger


logger = _get_logger(log_to_file=False)



def time_convert(spend_time):
    temp = spend_time
    second = round(temp % 60, 2)
    temp = temp // 60
    minute = int(temp % 60)
    temp = temp // 60
    hour = int(temp % 24)
    temp = temp // 24
    day = int(temp)
    format_str = ""
    if day > 0:
        format_str += "{}天".format(day)
    if hour > 0:
        format_str += "{}小时".format(hour)
    if minute > 0:
        format_str += "{}分钟".format(minute)
    format_str += "{}秒".format(second)
    return format_str



from .log import logger
from .time_format import time_convert
from .chunk import batch, cutoff_iter
from .file import check_file, get_file_name
from .variable import inspect_variable, retrieve_name
from .string import longest_common_subsequence, longest_common_substring
from .text_compare import string_compare
__all__ = ['logger', 'time_convert', 'batch', 'check_file', 'get_file_name', 'cutoff_iter', 'retrieve_name',
           'inspect_variable', 'longest_common_subsequence', 'longest_common_substring', 'string_compare']



"""
color.py
========

Usage
-----

>>> import color
>>>
>>> # 8-bit color
>>> print(red('red') + green('green') + blue('blue'))
>>> print(bold(yellow('bold yellow')) + underline(cyan('underline cyan')))
>>> print(magenta_hl('magenta highlight'))
>>>
>>> # xterm 256 color
>>> print(bg256('A9D5DE', fg256('276F86', 'Info!')))
>>> print(bg256('E0B4B4', fg256('912D2B', 'Warning!')))
>>> print(hl256('10a3a3', 'Teal'))

Note:

1. Every color function receives and returns string, so that the result
   could be used with any other strings, in any string formatting situation.

2. If you pass a str type string, the color function will return a str.
   If you pass a bytes type string, the color function will return a bytes string.

3. Color functions could be composed together, like put ``red`` into ``bold``,
   or put ``bg256`` into ``fg256``. ``xxx_hl`` and ``hl256`` are mostly used
   independently.

API
---

8-bit colors:

========  ============  ===========
 Colors    Background    Highlight
========  ============  ===========
black     black_bg      black_hl
red       red_bg        red_hl
green     green_bg      green_hl
yellow    yellow_bg     yellow_hl
blue      blue_bg       blue_hl
magenta   magenta_bg    magenta_hl
cyan      cyan_bg       cyan_hl
white     white_bg      white_hl
========  ============  ===========

Styles:
- bold
- italic
- underline
- strike
- blink

.. py:function:: <color_function>(s)

   Decorate string with specified color or style.

   A color function with ``_bg`` suffix means it will set color as background.
   A color function with ``_hl`` suffix means it will set color as background,
   and change the foreground as well to make the word standout.

   :param str s: The input string
   :return: The decorated string
   :rtype: string
   :raises ValueError: if the message_body exceeds 160 characters


256 colors:
- fg256
- bg256
- hl256

.. py:function:: <256_color_function>(hexrgb, s)

   Decorate string with specified hex rgb color

   ``fg256`` will set color as foreground.
   ``bg256`` will set color as background.
   ``hg256`` will highlight input with the color.

   :param str hexrgb: The hex rgb color string, accept length 3 and 6. eg: ``555``, ``912D2B``
   :param str s: The input string
   :return: The decorated string
   :rtype: string
   :raises ValueError: If the input string's length not equal to 3 or 6.
"""

from typing import Union, Any, Callable, Optional, Tuple, List, Dict
import sys


_use_color_no_tty = True


def use_color_no_tty(flag):
    global _use_color_no_tty
    _use_color_no_tty = flag


def use_color():
    if sys.stdout.isatty():
        return True
    if _use_color_no_tty:
        return True
    return False


def esc(*codes: Union[int, str]) -> str:
    """Produces an ANSI escape code from a list of integers
    :rtype: text_type
    """
    return t_('\x1b[{}m').format(t_(';').join(t_(str(c)) for c in codes))


def t_(b: Union[bytes, Any]) -> str:
    """ensure text type"""
    if isinstance(b, bytes):
        return b.decode()
    return b


def b_(t: Union[str, Any]) -> bytes:
    """ensure binary type"""
    if isinstance(t, str):
        return t.encode()
    return t


###############################################################################
# 8 bit Color
###############################################################################

def make_color(start, end: str) -> Callable[[str], str]:
    def color_func(s: str) -> str:
        if not use_color():
            return s

        # render
        return start + t_(s) + end

    return color_func


# According to https://en.wikipedia.org/wiki/ANSI_escape_code#graphics ,
# 39 is reset for foreground, 49 is reset for background, 0 is reset for all
# we can use 0 for convenience, but it will make color combination behaves weird.
END = esc(0)

FG_END = esc(39)
black = make_color(esc(30), FG_END)
red = make_color(esc(31), FG_END)
green = make_color(esc(32), FG_END)
yellow = make_color(esc(33), FG_END)
blue = make_color(esc(34), FG_END)
magenta = make_color(esc(35), FG_END)
cyan = make_color(esc(36), FG_END)
white = make_color(esc(37), FG_END)

BG_END = esc(49)
black_bg = make_color(esc(40), BG_END)
red_bg = make_color(esc(41), BG_END)
green_bg = make_color(esc(42), BG_END)
yellow_bg = make_color(esc(43), BG_END)
blue_bg = make_color(esc(44), BG_END)
magenta_bg = make_color(esc(45), BG_END)
cyan_bg = make_color(esc(46), BG_END)
white_bg = make_color(esc(47), BG_END)

HL_END = esc(22, 27, 39)
#HL_END = esc(22, 27, 0)

black_hl = make_color(esc(1, 30, 7), HL_END)
red_hl = make_color(esc(1, 31, 7), HL_END)
green_hl = make_color(esc(1, 32, 7), HL_END)
yellow_hl = make_color(esc(1, 33, 7), HL_END)
blue_hl = make_color(esc(1, 34, 7), HL_END)
magenta_hl = make_color(esc(1, 35, 7), HL_END)
cyan_hl = make_color(esc(1, 36, 7), HL_END)
white_hl = make_color(esc(1, 37, 7), HL_END)

bold = make_color(esc(1), esc(22))
italic = make_color(esc(3), esc(23))
underline = make_color(esc(4), esc(24))
strike = make_color(esc(9), esc(29))
blink = make_color(esc(5), esc(25))


###############################################################################
# Xterm 256 Color (delete if you don't need)
###############################################################################
#
# Rewrite from: https://gist.github.com/MicahElliott/719710

import re  # NOQA

# Default color levels for the color cube
CUBELEVELS: List[int] = [0x00, 0x5f, 0x87, 0xaf, 0xd7, 0xff]

# Generate a list of midpoints of the above list
SNAPS: List[int] = [(x + y) // 2 for x, y in list(zip(CUBELEVELS, [0] + CUBELEVELS))[1:]]

# Gray-scale range.
_GRAYSCALE = [
    (0x08, 232),  # 0x08 means 080808 in HEX color
    (0x12, 233),
    (0x1c, 234),
    (0x26, 235),
    (0x30, 236),
    (0x3a, 237),
    (0x44, 238),
    (0x4e, 239),
    (0x58, 240),
    (0x62, 241),
    (0x6c, 242),
    (0x76, 243),
    (0x80, 244),
    (0x8a, 245),
    (0x94, 246),
    (0x9e, 247),
    (0xa8, 248),
    (0xb2, 249),
    (0xbc, 250),
    (0xc6, 251),
    (0xd0, 252),
    (0xda, 253),
    (0xe4, 254),
    (0xee, 255),
]
GRAYSCALE: Dict[int, int] = dict(_GRAYSCALE)

GRAYSCALE_POINTS: List[int] = [i for i, _ in _GRAYSCALE]


def get_closest(v: int, l: list):
    return min(l, key=lambda x: abs(x - v))


class Memorize(dict):
    def __init__(self, func):
        self.func = func
        self.__doc__ = func.__doc__

    def __call__(self, *args):
        return self[args]

    def __missing__(self, key):
        result = self[key] = self.func(*key)
        return result


def memorize(func) -> Callable:
    func._cache = {}

    def wrapper(*args, **kwargs):
        if kwargs:
            return func(*args, **kwargs)
        if args not in func._cache:
            func._cache[args] = func(*args, **kwargs)
        return func._cache[args]

    for i in ('__module__', '__name__', '__doc__'):
        setattr(wrapper, i, getattr(func, i))
    wrapper.__dict__.update(getattr(func, '__dict__', {}))  # type: ignore
    wrapper._origin = func  # type: ignore
    return wrapper


@memorize
def rgb_to_xterm(r: int, g: int, b: int) -> int:
    """ Converts RGB values to the nearest equivalent xterm-256 color.
    """
    if r == g == b:
        # use gray scale
        gs = get_closest(r, GRAYSCALE_POINTS)
        return GRAYSCALE[gs]
    # Using list of snap points, convert RGB value to cube indexes
    r, g, b = map(lambda x: len(tuple(s for s in SNAPS if s < x)), (r, g, b))
    # Simple colorcube transform
    return r * 36 + g * 6 + b + 16


@memorize
def hex_to_rgb(hx: str) -> Tuple[int, int, int]:
    hxlen = len(hx)
    if hxlen != 3 and hxlen != 6:
        raise ValueError('hx color must be of length 3 or 6')
    if hxlen == 3:
        hx = t_('').join(i * 2 for i in hx)
    parts = [int(h, 16) for h in re.split(t_(r'(..)(..)(..)'), hx)[1:4]]
    return tuple(parts)  # type: ignore


def make_256(start: str, end: str) -> Callable[[Union[tuple, str], str, Optional[Tuple[int, int, int]]], str]:
    def rgb_func(rgb: Union[tuple, str], s: str, x: Optional[Tuple[int, int, int]] = None) -> str:
        """
        :param rgb: (R, G, B) tuple, or RRGGBB hex string
        """
        if not use_color():
            return s

        t = t_(s)

        # render
        if not isinstance(rgb, tuple):
            rgb = hex_to_rgb(t_(rgb))
        if x is not None:
            xcolor = x
        else:
            xcolor = rgb_to_xterm(*rgb)

        tpl = start + t_('{s}') + end
        f = tpl.format(
            x=xcolor,
            s=t)

        return f

    return rgb_func


fg256 = make_256(esc(38, 5, t_('{x}')), esc(39))
bg256 = make_256(esc(48, 5, t_('{x}')), esc(49))
hl256 = make_256(esc(1, 38, 5, t_('{x}'), 7), esc(27, 39, 22))

_grayscale_xterm_codes = [i for _, i in _GRAYSCALE]
grayscale = {(i - _grayscale_xterm_codes[0]): make_color(esc(38, 5, i), esc(39)) for i in _grayscale_xterm_codes}
grayscale_bg = {(i - _grayscale_xterm_codes[0]): make_color(esc(48, 5, i), esc(49)) for i in _grayscale_xterm_codes}
grayscale_hl = {(i - _grayscale_xterm_codes[0]): make_color(esc(1, 38, 5, i, 7), esc(27, 39, 22)) for i in _grayscale_xterm_codes}